==> jaguar_reflect-0.4.0.tar.gz: lib/src/interceptor/wrapper/wrapper_maker.dart <==
part of jaguar_reflect.base;

abstract class Wrapper {
  FutureOr<j.RouteWrapper> getWrapper(j.Context ctx, j.Response response);

  Type get interceptorType;

  _Pre get pre;

  _Post get post;
}

class MethodWrapper implements Wrapper {
  final InstanceMirror groupIm;

  final MethodMirror method;

  final ClosureMirror invoker;

  final List<Inject> injects;

  final Type interceptorType;

  final _Pre pre;

  final _Post post;

  MethodWrapper(this.groupIm, this.method, this.invoker, this.injects,
      this.interceptorType, this.pre, this.post);

  /// Creates wrapper from Wrapper maker method
  FutureOr<j.RouteWrapper> getWrapper(
      j.Context ctx, j.Response response) async {
    // Prepare parameters
    final List reqParam = [];

    for (Inject inj in injects) {
      reqParam.add(_makeParam(inj, ctx, response));
    }

    dynamic ret = invoker.apply(reqParam).reflectee;

    if (ret is Future) {
      ret = await ret;
    }

    if (ret is! j.RouteWrapper) {
      throw new Exception("Wrapper maker method ${method.simpleName} must return RouteWrapper");
    }

    return ret as j.RouteWrapper;
  }

  static MethodMirror _getMethod(Symbol sym, InstanceMirror groupIm) {
    final DeclarationMirror dm = groupIm.type.declarations[sym];

    if (dm is! MethodMirror) return null;

    return dm;
  }

  factory MethodWrapper.build(Symbol sym, InstanceMirror groupIm) {
    MethodMirror mm = _getMethod(sym, groupIm);

    //TODO what if return type is future
    if (!mm.returnType.isSubtypeOf(reflectType(j.RouteWrapper))) {
      throw new Exception("Wrapper maker method must return RouteWrapper");
    }

    _Pre pre;
    _Post post;
    ClassMirror icm;
    {
      icm = reflectClass(mm.returnType.reflectedType).superclass.typeArguments.first;
      DeclarationMirror preM = icm.declarations[#pre];
      if (preM is MethodMirror) {
        pre = new _Pre.build(preM);
      }
      DeclarationMirror postM = icm.declarations[#post];
      if (postM is MethodMirror) {
        post = new _Post.build(postM);
      }
    }

    final List<Inject> injects = _parseReqParamsForPost(mm);

    final cm = groupIm.getField(sym) as ClosureMirror;

    return new MethodWrapper(groupIm, mm, cm, injects, icm.reflectedType, pre, post);
  }
}

==> jaguar_reflect-0.4.2.tar.gz: lib/src/interceptor/wrapper/wrapper_maker.dart <==
part of jaguar_reflect.base;

abstract class Wrapper {
  FutureOr<j.RouteWrapper> getWrapper(j.Context ctx, j.Response response);

  Type get interceptorType;

  _Pre get pre;

  _Post get post;
}

class MethodWrapper implements Wrapper {
  final InstanceMirror groupIm;

  final MethodMirror method;

  final ClosureMirror invoker;

  final List<Inject> injects;

  final Type interceptorType;

  final _Pre pre;

  final _Post post;

  MethodWrapper(this.groupIm, this.method, this.invoker, this.injects,
      this.interceptorType, this.pre, this.post);

  /// Creates wrapper from Wrapper maker method
  FutureOr<j.RouteWrapper> getWrapper(
      j.Context ctx, j.Response response) async {
    // Prepare parameters
    final List reqParam = [];

    for (Inject inj in injects) {
      reqParam.add(_makeParam(inj, ctx, response));
    }

    dynamic ret = invoker.apply(reqParam).reflectee;

    if (ret is Future) {
      ret = await ret;
    }

    if (ret is! j.RouteWrapper) {
      throw new Exception(
          "Wrapper maker method ${method.simpleName} must return RouteWrapper");
    }

    return ret as j.RouteWrapper;
  }

  static MethodMirror _getMethod(Symbol sym, InstanceMirror groupIm) {
    final DeclarationMirror dm = groupIm.type.declarations[sym];

    if (dm is! MethodMirror) return null;

    return dm;
  }

  factory MethodWrapper.build(Symbol sym, InstanceMirror groupIm) {
    MethodMirror mm = _getMethod(sym, groupIm);

    //TODO what if return type is future
    if (!mm.returnType.isSubtypeOf(reflectType(j.RouteWrapper))) {
      throw new Exception("Wrapper maker method must return RouteWrapper");
    }

    _Pre pre;
    _Post post;
    ClassMirror icm;
    {
      icm = reflectClass(mm.returnType.reflectedType)
          .superclass
          .typeArguments
          .first;
      DeclarationMirror preM = icm.declarations[#pre];
      if (preM is MethodMirror) {
        pre = new _Pre.build(preM);
      }
      DeclarationMirror postM = icm.declarations[#post];
      if (postM is MethodMirror) {
        post = new _Post.build(postM);
      }
    }

    final List<Inject> injects = _parseReqParamsForPost(mm);

    final cm = groupIm.getField(sym) as ClosureMirror;

    return new MethodWrapper(
        groupIm, mm, cm, injects, icm.reflectedType, pre, post);
  }
}
